@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2005-2008 Sun Microsystems Inc.
+# Copyright 2005-2009 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -25,10 +25,10 @@ operations on the Accessible object, such as invoking default actions or
 moving the text caret.
 """
 
-__id__        = "$Id: braille.py 4273 2008-10-03 17:35:13Z joanied $"
-__version__   = "$Revision: 4273 $"
-__date__      = "$Date: 2008-10-03 13:35:13 -0400 (Fri, 03 Oct 2008) $"
-__copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
+__id__        = "$Id$"
+__version__   = "$Revision$"
+__date__      = "$Date$"
+__copyright__ = "Copyright (c) 2005-2009 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import logging
@@ -44,21 +44,15 @@ except ImportError:
 else:
     _defaultContractionTable = louis.getDefaultTable()
 
-# We'll use the official BrlAPI pythons (as of BrlTTY 3.8) if they
-# are available.  Otherwise, we'll fall back to our own bindings.
-#
 try:
     import brlapi
     import gobject
 
-    brlAPI = None
-    useBrlAPIBindings = True
-    brlAPIRunning = False
-    brlAPISourceId = 0
+    _brlAPI = None
+    _brlAPIRunning = False
+    _brlAPISourceId = 0
 except:
-    import brl
-    useBrlAPIBindings = False
-    brlAPIRunning = False
+    _brlAPIRunning = False
 
 try:
     # This can fail due to gtk not being available.  We want to
@@ -84,82 +78,6 @@ _initialized = False
 #
 monitor = None
 
-# Each of these maps to BrlAPI's brldefs.h file.
-#
-CMD_NOOP              = 0x00
-CMD_LNUP              = 0x01
-CMD_LNDN              = 0x02
-CMD_WINUP             = 0x03
-CMD_WINDN             = 0x04
-CMD_PRDIFLN           = 0x05
-CMD_NXDIFLN           = 0x06
-CMD_ATTRUP            = 0x07
-CMD_ATTRDN            = 0x08
-CMD_TOP               = 0x09
-CMD_BOT               = 0x0a
-CMD_TOP_LEFT          = 0x0b
-CMD_BOT_LEFT          = 0x0c
-CMD_PRPGRPH           = 0x0d
-CMD_NXPGRPH           = 0x0e
-CMD_PRPROMPT          = 0x0f
-CMD_NXPROMPT          = 0x10
-CMD_PRSEARCH          = 0x11
-CMD_NXSEARCH          = 0x12
-CMD_CHRLT             = 0x13
-CMD_CHRRT             = 0x14
-CMD_HWINLT            = 0x15
-CMD_HWINRT            = 0x16
-CMD_FWINLT            = 0x17
-CMD_FWINRT            = 0x18
-CMD_FWINLTSKIP        = 0x19
-CMD_FWINRTSKIP        = 0x1a
-CMD_LNBEG             = 0x1b
-CMD_LNEND             = 0x1c
-CMD_HOME              = 0x1d
-CMD_BACK              = 0x1e
-CMD_FREEZE            = 0x1f
-CMD_DISPMD            = 0x20
-CMD_SIXDOTS           = 0x21
-CMD_SLIDEWIN          = 0x22
-CMD_SKPIDLNS          = 0x23
-CMD_SKPBLNKWINS       = 0x24
-CMD_CSRVIS            = 0x25
-CMD_CSRHIDE           = 0x26
-CMD_CSRTRK            = 0x27
-CMD_CSRSIZE           = 0x28
-CMD_CSRBLINK          = 0x29
-CMD_ATTRVIS           = 0x2a
-CMD_ATTRBLINK         = 0x2b
-CMD_CAPBLINK          = 0x2c
-CMD_TUNES             = 0x2d
-CMD_HELP              = 0x2e
-CMD_INFO              = 0x2f
-CMD_LEARN             = 0x30
-CMD_PREFMENU          = 0x31
-CMD_PREFSAVE          = 0x32
-CMD_PREFLOAD          = 0x33
-CMD_MENU_FIRST_ITEM   = 0x34
-CMD_MENU_LAST_ITEM    = 0x35
-CMD_MENU_PREV_ITEM    = 0x36
-CMD_MENU_NEXT_ITEM    = 0x37
-CMD_MENU_PREV_SETTING = 0x38
-CMD_MENU_NEXT_SETTING = 0x39
-CMD_SAY_LINE          = 0x3a
-CMD_SAY_ABOVE         = 0x3b
-CMD_SAY_BELOW         = 0x3c
-CMD_MUTE              = 0x3d
-CMD_SPKHOME           = 0x3e
-CMD_SWITCHVT_PREV     = 0x3f
-CMD_SWITCHVT_NEXT     = 0x40
-CMD_CSRJMP_VERT       = 0x41
-CMD_PASTE             = 0x42
-CMD_RESTARTBRL        = 0x43
-CMD_RESTARTSPEECH     = 0x44
-CMD_MAX               = 0x44
-
-BRL_FLG_REPEAT_INITIAL = 0x800000
-BRL_FLG_REPEAT_DELAY   = 0x400000
-
 # Common names for most used BrlTTY commands, to be shown in the GUI:
 # ATM, the ones used in default.py are:
 #
@@ -169,46 +87,85 @@ command_name = {}
 # display (an external hardware device used by people who are blind).
 # When pressing the button, the display scrolls to the left.
 #
-command_name[CMD_FWINLT]   = _("Line Left")
+command_name[brlapi.KEY_CMD_FWINLT]   = _("Line Left")
 
 # Translators: this is a command for a button on a refreshable braille
 # display (an external hardware device used by people who are blind).
 # When pressing the button, the display scrolls to the right.
 #
-command_name[CMD_FWINRT]   = _("Line Right")
+command_name[brlapi.KEY_CMD_FWINRT]   = _("Line Right")
 
 # Translators: this is a command for a button on a refreshable braille
 # display (an external hardware device used by people who are blind).
 # When pressing the button, the display scrolls up.
 #
-command_name[CMD_LNUP]     = _("Line Up")
+command_name[brlapi.KEY_CMD_LNUP]     = _("Line Up")
 
 # Translators: this is a command for a button on a refreshable braille
 # display (an external hardware device used by people who are blind).
 # When pressing the button, the display scrolls down.
 #
-command_name[CMD_LNDN]     = _("Line Down")
+command_name[brlapi.KEY_CMD_LNDN]     = _("Line Down")
+
+# Translators: this is a command for a button on a refreshable braille
+# display (an external hardware device used by people who are blind).
+# When pressing the button, it instructs the braille display to freeze.
+#
+command_name[brlapi.KEY_CMD_FREEZE]     = _("Freeze")
 
 # Translators: this is a command for a button on a refreshable braille
 # display (an external hardware device used by people who are blind).
 # When pressing the button, the display scrolls to the top left of the
 # window.
 #
-command_name[CMD_TOP_LEFT] = _("Top Left")
+command_name[brlapi.KEY_CMD_TOP_LEFT] = _("Top Left")
 
 # Translators: this is a command for a button on a refreshable braille
 # display (an external hardware device used by people who are blind).
 # When pressing the button, the display scrolls to the bottom right of
 # the window.
 #
-command_name[CMD_BOT_LEFT] = _("Bottom Right")
+command_name[brlapi.KEY_CMD_BOT_LEFT] = _("Bottom Right")
 
 # Translators: this is a command for a button on a refreshable braille
 # display (an external hardware device used by people who are blind).
 # When pressing the button, the display scrolls to position containing
 # the cursor.
 #
-command_name[CMD_HOME]     = _("Cursor Position")
+command_name[brlapi.KEY_CMD_HOME]     = _("Cursor Position")
+
+# Translators: this is a command for a button on a refreshable braille
+# display (an external hardware device used by people who are blind).
+# When pressing the button, the display toggles between contracted and
+# contracted braille.
+#
+command_name[brlapi.KEY_CMD_SIXDOTS]  = _("Six Dots")
+
+# Translators: this is a command for a button on a refreshable braille
+# display (an external hardware device used by people who are blind).
+# This command represents a whole set of buttons known as cursor
+# routings keys and are a way for a user to tell the machine they are
+# interested in a particular character on the display.
+#
+command_name[brlapi.KEY_CMD_ROUTE]    = _("Cursor Routing")
+
+# Translators: this is a command for a button on a refreshable braille
+# display (an external hardware device used by people who are blind).
+# This command represents the start of a selection operation.  It is
+# called "Cut Begin" to map to what BrlTTY users are used to:  in
+# character cell mode operation on virtual consoles, the act of copying
+# text is erroneously called a "cut" operation.
+#
+command_name[brlapi.KEY_CMD_CUTBEGIN] = _("Cut Begin")
+
+# Translators: this is a command for a button on a refreshable braille
+# display (an external hardware device used by people who are blind).
+# This command represents marking the endpoint of a selection.  It is
+# called "Cut Line" to map to what BrlTTY users are used to:  in
+# character cell mode operation on virtual consoles, the act of copying
+# text is erroneously called a "cut" operation.
+#
+command_name[brlapi.KEY_CMD_CUTLINE] = _("Cut Line")
 
 # The size of the physical display (width, height).  The coordinate system of
 # the display is set such that the upper left is (0,0), x values increase from
@@ -263,7 +220,7 @@ def _printBrailleEvent(level, command):
 
     debug.printInputEvent(
         level,
-        "BRAILLE EVENT: %x" % command)
+        "BRAILLE EVENT: %s" % repr(command))
 
 class Region:
     """A Braille region to be displayed on the display.  The width of
@@ -285,9 +242,9 @@ class Region:
         # If louis is None, then we don't go into contracted mode.
         self.contracted = settings.enableContractedBraille and \
                           louis is not None
-        
+
         self.expandOnCursor = expandOnCursor
-        
+
         # The uncontracted string for the line.
         #
         self.rawLine = string.decode("UTF-8").strip("\n")
@@ -302,7 +259,7 @@ class Region:
         else:
             self.string = self.rawLine
             self.cursorOffset = cursorOffset
-            
+
     def processRoutingKey(self, offset):
         """Processes a cursor routing key press on this Component.  The offset
         is 0-based, where 0 represents the leftmost character of string
@@ -330,7 +287,7 @@ class Region:
         #
         mask = ['\x00'] * maskSize
         return "".join(mask)
-    
+
     def repositionCursor(self):
         """Reposition the cursor offset for contracted mode.
         """
@@ -354,7 +311,7 @@ class Region:
             cursorOnSpace = line[cursorOffset] == ' '
         except IndexError:
             cursorOnSpace = False
-            
+
         if not expandOnCursor or cursorOnSpace:
             contracted, inPos, outPos, cursorPos = \
                              louis.translate([self.contractionTable],
@@ -368,7 +325,7 @@ class Region:
                                              mode=louis.MODE.compbrlAtCursor)
 
         return contracted, inPos, outPos, cursorPos
-    
+
     def displayToBufferOffset(self, display_offset):
         try:
             offset = self.inPos[display_offset]
@@ -382,9 +339,6 @@ class Region:
         return offset
 
     def setContractedBraille(self, contracted):
-        if self.contracted == contracted:
-            return
-        self.contracted = contracted
         if contracted:
             self.contractionTable = settings.brailleContractionTable or \
                                     _defaultContractionTable
@@ -393,11 +347,14 @@ class Region:
             self.expandRegion()
 
     def contractRegion(self):
+        if self.contracted:
+            return
         self.string, self.inPos, self.outPos, self.cursorOffset = \
                      self.contractLine(self.rawLine,
                                        self.cursorOffset,
                                        self.expandOnCursor)
-        
+        self.contracted = True
+
     def expandRegion(self):
         if not self.contracted:
             return
@@ -406,7 +363,8 @@ class Region:
             self.cursorOffset = self.inPos[self.cursorOffset]
         except IndexError:
             self.cursorOffset = len(self.string)
-        
+        self.contracted = False
+
 class Component(Region):
     """A subclass of Region backed by an accessible.  This Region will react
     to any cursor routing key events and perform the default action on the
@@ -433,6 +391,15 @@ class Component(Region):
 
         self.accessible = accessible
 
+    def getCaretOffset(self, offset):
+        """Returns the caret position of the given offset if the object
+        has text with a caret.  Otherwise, returns -1.
+
+        Arguments:
+        - offset: 0-based offset of the cell on the physical display
+        """
+        return -1
+
     def processRoutingKey(self, offset):
         """Processes a cursor routing key press on this Component.  The offset
         is 0-based, where 0 represents the leftmost character of string
@@ -489,7 +456,7 @@ class Text(Region):
     [[[TODO: WDW - need to add in text selection capabilities.  Logged
     as bugzilla bug 319754.]]]"""
 
-    def __init__(self, accessible, label="", eol="", 
+    def __init__(self, accessible, label="", eol="",
                  startOffset=None, endOffset=None):
         """Creates a new Text region.
 
@@ -559,7 +526,7 @@ class Text(Region):
         string = string.decode("UTF-8")
 
         cursorOffset = min(caretOffset - lineOffset, len(string))
-        
+
         if lineOffset != self.lineOffset:
             return False
 
@@ -576,20 +543,33 @@ class Text(Region):
 
         return True
 
+    def getCaretOffset(self, offset):
+        """Returns the caret position of the given offset if the object
+        has text with a caret.  Otherwise, returns -1.
+
+        Arguments:
+        - offset: 0-based offset of the cell on the physical display
+        """
+        offset = self.displayToBufferOffset(offset)
+
+        if offset < 0:
+            return -1
+
+        return min(self.lineOffset + offset, self._maxCaretOffset)
+
     def processRoutingKey(self, offset):
         """Processes a cursor routing key press on this Component.  The offset
         is 0-based, where 0 represents the leftmost character of text
         associated with this region.  Note that the zeroeth character may have
-        been scrolled off the display."""
-        
-        offset = self.displayToBufferOffset(offset)
+        been scrolled off the display.
+        """
 
-        if offset < 0:
+        caretOffset = self.getCaretOffset(offset)
+
+        if caretOffset < 0:
             return
 
-        newCaretOffset = min(self.lineOffset + offset, self._maxCaretOffset)
-        orca_state.activeScript.setCaretOffset(
-            self.accessible, newCaretOffset)
+        orca_state.activeScript.setCaretOffset(self.accessible, caretOffset)
 
     def getAttributeMask(self, getLinkMask=True):
         """Creates a string which can be used as the attrOr field of brltty's
@@ -696,7 +676,7 @@ class Text(Region):
     def contractLine(self, line, cursorOffset=0, expandOnCursor=True):
         contracted, inPos, outPos, cursorPos = Region.contractLine(
             self, line, cursorOffset, expandOnCursor)
-        
+
         return contracted + self.eol, inPos, outPos, cursorPos
 
     def displayToBufferOffset(self, display_offset):
@@ -748,15 +728,28 @@ class ReviewText(Region):
         self.lineOffset = lineOffset
         self.zone = zone
 
+    def getCaretOffset(self, offset):
+        """Returns the caret position of the given offset if the object
+        has text with a caret.  Otherwise, returns -1.
+
+        Arguments:
+        - offset: 0-based offset of the cell on the physical display
+        """
+        offset = self.displayToBufferOffset(offset)
+
+        if offset < 0:
+            return -1
+
+        return self.lineOffset + offset
+
     def processRoutingKey(self, offset):
         """Processes a cursor routing key press on this Component.  The offset
         is 0-based, where 0 represents the leftmost character of text
         associated with this region.  Note that the zeroeth character may have
         been scrolled off the display."""
 
-        offset = self.displayToBufferOffset(offset)
-        newCaretOffset = self.lineOffset + offset
-        orca_state.activeScript.setCaretOffset(self.accessible, newCaretOffset)
+        caretOffset = self.getCaretOffset(offset)
+        orca_state.activeScript.setCaretOffset(self.accessible, caretOffset)
 
 class Line:
     """A horizontal line on the display.  Each Line is composed of a sequential
@@ -854,7 +847,8 @@ def getRegionAtCell(cell):
     associated with that cell in the form of [region, offsetinregion]
     where 'region' is the region associated with the cell and
     'offsetinregion' is the 0-based offset of where the cell is
-    in the region, where 0 represents the beginning of the region, """
+    in the region, where 0 represents the beginning of the region.
+    """
 
     if len(_lines) > 0:
         offset = (cell - 1) + viewport[0]
@@ -863,6 +857,27 @@ def getRegionAtCell(cell):
     else:
         return [None, -1]
 
+def getCaretContext(event):
+    """Gets the accesible and caret offset associated with the given
+    event.  The event should have a BrlAPI event that contains an
+    argument value that corresponds to a cell on the display.
+
+    Arguments:
+    - event: an instance of input_event.BrailleEvent.  event.event is
+    the dictionary form of the expanded BrlAPI event.
+    """
+
+    offset = event.event["argument"]
+    [region, regionOffset] = getRegionAtCell(offset + 1)
+    if region and (isinstance(region, Text) or isinstance(region, ReviewText)):
+        accessible = region.accessible
+        caretOffset = region.getCaretOffset(regionOffset)
+    else:
+        accessible = None
+        caretOffset = -1
+
+    return [accessible, caretOffset]
+
 def clear():
     """Clears the logical structure, but keeps the Braille display as is
     (until a refresh operation).
@@ -980,11 +995,8 @@ def refresh(panToCursor=True, targetCursorCell=0, getLinkMask=True):
     global monitor
 
     if len(_lines) == 0:
-        if useBrlAPIBindings:
-            if brlAPIRunning:
-                brlAPI.writeText("", 0)
-        else:
-            brl.writeText(0, "")
+        if _brlAPIRunning:
+            _brlAPI.writeText("", 0)
         return
 
     # Now determine the location of the cursor.  First, we'll figure
@@ -1040,40 +1052,37 @@ def refresh(panToCursor=True, targetCursorCell=0, getLinkMask=True):
     log.info(logLine.encode("UTF-8"))
 
     substring = string[startPos:endPos]
-    if useBrlAPIBindings:
-        if brlAPIRunning:
-            writeStruct = brlapi.WriteStruct()
-            writeStruct.regionBegin = 1
-            writeStruct.regionSize = len(substring)
-            while writeStruct.regionSize < _displaySize[0]:
-                substring += " "
-                if attributeMask:
-                    attributeMask += '\x00'
-                writeStruct.regionSize += 1
-            writeStruct.text = substring
-            writeStruct.cursor = cursorCell
-
-            # [[[WDW - if you want to muck around with the dots on the
-            # display to do things such as add underlines, you can use
-            # the attrOr field of the write structure to do so.  The
-            # attrOr field is a string whose length must be the same
-            # length as the display and whose dots will end up showing
-            # up on the display.  Each character represents a bitfield
-            # where each bit corresponds to a dot (i.e., bit 0 = dot 1,
-            # bit 1 = dot 2, and so on).  Here's an example that underlines
-            # all the text.]]]
-            #
-            #myUnderline = ""
-            #for i in range(0, _displaySize[0]):
-            #    myUnderline += '\xc0'
-            #writeStruct.attrOr = myUnderline
-
+    if _brlAPIRunning:
+        writeStruct = brlapi.WriteStruct()
+        writeStruct.regionBegin = 1
+        writeStruct.regionSize = len(substring)
+        while writeStruct.regionSize < _displaySize[0]:
+            substring += " "
             if attributeMask:
-                writeStruct.attrOr = attributeMask[startPos:endPos]
+                attributeMask += '\x00'
+            writeStruct.regionSize += 1
+        writeStruct.text = substring
+        writeStruct.cursor = cursorCell
 
-            brlAPI.write(writeStruct)
-    else:
-        brl.writeText(cursorCell, substring)
+        # [[[WDW - if you want to muck around with the dots on the
+        # display to do things such as add underlines, you can use
+        # the attrOr field of the write structure to do so.  The
+        # attrOr field is a string whose length must be the same
+        # length as the display and whose dots will end up showing
+        # up on the display.  Each character represents a bitfield
+        # where each bit corresponds to a dot (i.e., bit 0 = dot 1,
+        # bit 1 = dot 2, and so on).  Here's an example that underlines
+        # all the text.]]]
+        #
+        #myUnderline = ""
+        #for i in range(0, _displaySize[0]):
+        #    myUnderline += '\xc0'
+        #writeStruct.attrOr = myUnderline
+
+        if attributeMask:
+            writeStruct.attrOr = attributeMask[startPos:endPos]
+
+        _brlAPI.write(writeStruct)
 
     if settings.enableBrailleMonitor:
         if not monitor:
@@ -1204,29 +1213,46 @@ def returnToRegionWithFocus(inputEvent=None):
 
     return True
 
-def _processBrailleEvent(command):
-    """Handles BrlTTY command events.  This passes commands on to Orca for
-    processing.  If Orca does not handle them (as indicated by a return value
-    of false from the callback passed to init, it will attempt to handle the
-    command itself - either by panning the viewport or passing cursor routing
-    keys to the Regions for handling.
+def setContractedBraille(event):
+    """Turns contracted braille on or off based upon the event.
 
     Arguments:
-    - command: the BrlAPI command for the key that was pressed.
+    - event: an instance of input_event.BrailleEvent.  event.event is
+    the dictionary form of the expanded BrlAPI event.
     """
 
-    _printBrailleEvent(debug.LEVEL_FINE, command)
+    settings.enableContractedBraille = \
+        (event.event["flags"] & brlapi.KEY_FLG_TOGGLE_ON) != 0
+    for line in _lines:
+        line.setContractedBraille(settings.enableContractedBraille)
+    refresh()
 
-    # [[[TODO: WDW - DaveM suspects the Alva driver is sending us a
-    # repeat flag.  So...let's kill a couple birds here until BrlTTY
-    # 3.8 fixes the problem: we'll disable autorepeat and we'll also
-    # strip out the autorepeat flag if this is the first press of a
-    # button.]]]
-    #
-    if command & BRL_FLG_REPEAT_INITIAL:
-        command &= ~(BRL_FLG_REPEAT_INITIAL | BRL_FLG_REPEAT_DELAY)
-    elif command & BRL_FLG_REPEAT_DELAY:
-        return True
+def processRoutingKey(event):
+    """Processes a cursor routing key event.
+
+    Arguments:
+    - event: an instance of input_event.BrailleEvent.  event.event is
+    the dictionary form of the expanded BrlAPI event.
+    """
+
+    cell = event.event["argument"]
+
+    if len(_lines) > 0:
+        cursor = cell + viewport[0]
+        lineNum = viewport[1]
+        _lines[lineNum].processRoutingKey(cursor)
+
+    return True
+
+def _processBrailleEvent(event):
+    """Handles BrlTTY command events.  This passes commands on to Orca for
+    processing.
+
+    Arguments:
+    - event: the BrlAPI input event (expanded)
+    """
+
+    _printBrailleEvent(debug.LEVEL_FINE, event)
 
     consumed = False
 
@@ -1239,24 +1265,11 @@ def _processBrailleEvent(command):
             # Like key event handlers, a return value of True means
             # the command was consumed.
             #
-            consumed = _callback(command)
+            consumed = _callback(event)
         except:
             debug.printException(debug.LEVEL_WARNING)
             consumed = False
 
-    if (command >= 0x100) and (command < (0x100 + _displaySize[0])):
-        if len(_lines) > 0:
-            cursor = (command - 0x100) + viewport[0]
-            lineNum = viewport[1]
-            _lines[lineNum].processRoutingKey(cursor)
-            consumed = True
-
-    if command in (0x2141, 65): # Toggle six dot braille
-        settings.enableContractedBraille = not settings.enableContractedBraille
-        for line in _lines:
-            line.setContractedBraille(settings.enableContractedBraille)
-        refresh()
-
     if settings.timeoutCallback and (settings.timeoutTime > 0):
         signal.alarm(0)
 
@@ -1266,24 +1279,10 @@ def _brlAPIKeyReader(source, condition):
     """Method to read a key from the BrlAPI bindings.  This is a
     gobject IO watch handler.
     """
-    key = brlAPI.readKey(False)
+    key = _brlAPI.readKey(False)
     if key:
-        #flags = key >> 32
-        lower = key & 0xFFFFFFFF
-        #keyType = lower >> 29
-        keyCode = lower & 0x1FFFFFFF
-        
-        # [[TODO: WDW - HACK If we have a cursor routing key, map
-        # it back to the code we used to get with earlier versions
-        # of BrlAPI (i.e., bit 0x100 was the indicator of a cursor
-        # routing key instead of 0x1000).  This may change before
-        # the offical BrlAPI Python bindings are released.]]]
-        #
-        if keyCode & 0x10000:
-            keyCode = 0x100 | (keyCode & 0xFF)
-        if keyCode:
-            _processBrailleEvent(keyCode)
-    return brlAPIRunning
+        _processBrailleEvent(_brlAPI.expandKeyCode(key))
+    return _brlAPIRunning
 
 def setupKeyRanges(keys):
     """Hacky method to tell BrlTTY what to send and not send us via
@@ -1292,64 +1291,23 @@ def setupKeyRanges(keys):
     Arguments:
     -keys: a list of BrlAPI commands.
     """
-    if not brlAPIRunning:
+    if not _brlAPIRunning:
         return
 
-    try:
-        # First, start by ignoring everything.
-        #
-        brlAPI.ignoreKeys(brlapi.rangeType_all, [0])
+    # First, start by ignoring everything.
+    #
+    _brlAPI.ignoreKeys(brlapi.rangeType_all, [0])
 
-        # Next, enable cursor routing keys.
-        #
-        keySet = [brlapi.KEY_TYPE_CMD | brlapi.KEY_CMD_ROUTE]
+    # Next, enable cursor routing keys.
+    #
+    keySet = [brlapi.KEY_TYPE_CMD | brlapi.KEY_CMD_ROUTE]
 
-        # Finally, enable the commands we care about.
-        #
-        for key in keys:
-            keySet.append(brlapi.KEY_TYPE_CMD | key)
-            
-        brlAPI.acceptKeys(brlapi.rangeType_command, keySet)
+    # Finally, enable the commands we care about.
+    #
+    for key in keys:
+        keySet.append(brlapi.KEY_TYPE_CMD | key)
 
-        brlAPI.acceptKeys(brlapi.rangeType_key, [65])
-
-        debug.println(debug.LEVEL_FINEST, "Using BrlAPI v0.5.0+")
-    except:
-        debug.printException(debug.LEVEL_FINEST)
-        try:
-            # Old, incompatible way that was in v3.8 devel, but
-            # changed prior to release.  We need this just in case
-            # people have not updated yet.
-
-            # First, start by ignoring everything.
-            #
-            brlAPI.ignoreKeyRange(0,
-                                  brlapi.KEY_FLAGS_MASK \
-                                  | brlapi.KEY_TYPE_MASK \
-                                  | brlapi.KEY_CODE_MASK)
-
-            # Next, enable cursor routing keys.
-            #
-            brlAPI.acceptKeyRange(brlapi.KEY_TYPE_CMD | brlapi.KEY_CMD_ROUTE,
-                                  brlapi.KEY_TYPE_CMD \
-                                  | brlapi.KEY_CMD_ROUTE \
-                                  | brlapi.KEY_CMD_ARG_MASK)
-
-            # Finally, enable the commands we care about.
-            #
-            keySet = []
-            for key in keys:
-                keySet.append(brlapi.KEY_TYPE_CMD | key)
-            if len(keySet):
-                brlAPI.acceptKeySet(keySet)
-
-            debug.println(debug.LEVEL_FINEST,
-                          "Using BrlAPI pre-release v0.5.0")
-        except:
-            debug.printException(debug.LEVEL_FINEST)
-            debug.println(
-                debug.LEVEL_WARNING,
-                "Braille module cannot listen for braille input events")
+    _brlAPI.acceptKeys(brlapi.rangeType_command, keySet)
 
 def init(callback=None, tty=7):
     """Initializes the braille module, connecting to the BrlTTY driver.
@@ -1370,55 +1328,42 @@ def init(callback=None, tty=7):
 
     _callback = callback
 
-    if useBrlAPIBindings:
+    try:
+        global _brlAPI
+        global _brlAPIRunning
+        global _brlAPISourceId
+
+        gobject.threads_init()
+        _brlAPI = brlapi.Connection()
+
         try:
-            global brlAPI
-            global brlAPIRunning
-            global brlAPISourceId
-
-            gobject.threads_init()
-            brlAPI = brlapi.Connection()
-
-            try:
-                import os
-                windowPath = os.environ["WINDOWPATH"]
-                brlAPI.enterTtyModeWithPath()
-                brlAPIRunning = True
-                debug.println(\
-                    debug.LEVEL_CONFIGURATION,
-                    "Braille module has been initialized using WINDOWPATH=" \
-                    + "%s" % windowPath)
-            except:
-                brlAPI.enterTtyMode(tty)
-                brlAPIRunning = True
-                debug.println(\
-                    debug.LEVEL_CONFIGURATION,
-                    "Braille module has been initialized using tty=%d" % tty)
-            brlAPISourceId = gobject.io_add_watch(brlAPI.fileDescriptor,
-                                                  gobject.IO_IN,
-                                                  _brlAPIKeyReader)
+            import os
+            windowPath = os.environ["WINDOWPATH"]
+            _brlAPI.enterTtyModeWithPath()
+            _brlAPIRunning = True
+            debug.println(\
+                debug.LEVEL_CONFIGURATION,
+                "Braille module has been initialized using WINDOWPATH=" \
+                + "%s" % windowPath)
         except:
-            debug.printException(debug.LEVEL_FINEST)
-            return False
-    else:
-        if brl.init(tty):
-            debug.println(debug.LEVEL_CONFIGURATION,
-                          "Braille module has been initialized.")
-            brl.registerCallback(_processBrailleEvent)
-        else:
-            debug.println(debug.LEVEL_CONFIGURATION,
-                          "Braille module has NOT been initialized.")
-            return False
+            _brlAPI.enterTtyMode(tty)
+            _brlAPIRunning = True
+            debug.println(\
+                debug.LEVEL_CONFIGURATION,
+                "Braille module has been initialized using tty=%d" % tty)
+        _brlAPISourceId = gobject.io_add_watch(_brlAPI.fileDescriptor,
+                                               gobject.IO_IN,
+                                               _brlAPIKeyReader)
+    except:
+        debug.printException(debug.LEVEL_FINEST)
+        return False
 
     # [[[TODO: WDW - For some reason, BrlTTY wants to say the height of the
     # Vario is 40 so we hardcode it to 1 for now.]]]
     #
     #_displaySize = (brl.getDisplayWidth(), brl.getDisplayHeight())
-    if useBrlAPIBindings:
-        (x, y) = brlAPI.displaySize
-        _displaySize = [x, 1]
-    else:
-        _displaySize = [brl.getDisplayWidth(), 1]
+    (x, y) = _brlAPI.displaySize
+    _displaySize = [x, 1]
 
     debug.println(debug.LEVEL_CONFIGURATION,
                   "braille display size = (%d, %d)" \
@@ -1441,17 +1386,14 @@ def shutdown():
     if not _initialized:
         return False
 
-    global brlAPIRunning
-    global brlAPISourceId
+    global _brlAPIRunning
+    global _brlAPISourceId
 
-    if useBrlAPIBindings:
-        if brlAPIRunning:
-            brlAPIRunning = False
-            gobject.source_remove(brlAPISourceId)
-            brlAPISourceId = 0
-            brlAPI.leaveTtyMode()
-    else:
-        brl.shutdown()
+    if _brlAPIRunning:
+        _brlAPIRunning = False
+        gobject.source_remove(_brlAPISourceId)
+        _brlAPISourceId = 0
+        _brlAPI.leaveTtyMode()
 
     _initialized = False
 
