@@ -23,9 +23,9 @@ implementation in Gecko:
 http://developer.mozilla.org/en/docs/Accessibility/ATSPI_Support
 """
 
-__id__        = "$Id: where_am_i.py 4249 2008-09-21 01:34:35Z joanied $"
-__version__   = "$Revision: 4249 $"
-__date__      = "$Date: 2008-09-20 21:34:35 -0400 (Sat, 20 Sep 2008) $"
+__id__        = "$Id$"
+__version__   = "$Revision$"
+__date__      = "$Date$"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -103,6 +103,23 @@ class GeckoWhereAmI(where_am_I.WhereAmI):
 
         return where_am_I.WhereAmI._getSpeechForRoleName(self, obj, role)
 
+    def _getObjName(self, obj):
+        """Returns the name to speak for an object.
+        """
+
+        text = ""
+        name = self._script.getDisplayedText(obj)
+        if not name:
+            name = obj.description
+            if not name and obj.getRole() == pyatspi.ROLE_LIST_ITEM:
+                name = self._script.expandEOCs(obj)
+
+        if name and name != "None":
+            text = name.strip()
+        debug.println(self._debugLevel, "%s name=<%s>" % (obj.getRole(), text))
+
+        return text
+
     def _speakObjDescription(self, obj):
         """Speaks the object's description if it is not the same as the
         object's name or label. Overridden here because Gecko tacks on
